@@ -46,6 +46,7 @@ module LLVM.Core(
     zero, allOnes, undef,
     createString, createStringNul,
     withString, withStringNul,
+    withModule, Module(..),
     --constString, constStringNul,
     constVector, constArray,
     constStruct, constPackedStruct,
@@ -64,7 +65,7 @@ module LLVM.Core(
     -- * Globals
     Linkage(..),
     -- * Basic blocks
-    BasicBlock, newBasicBlock, newNamedBasicBlock, defineBasicBlock, createBasicBlock, getCurrentBasicBlock,
+    BasicBlock, newBasicBlock, newNamedBasicBlock, defineBasicBlock, createBasicBlock, createNamedBasicBlock, getCurrentBasicBlock,
     getBasicBlocks,              
     fromLabel, toLabel,
     getInstructions, getOperands, hasUsers, getUsers, getUses, getUser, isChildOf, getDep,
@@ -72,8 +73,10 @@ module LLVM.Core(
     addAttributes, Attribute(..),
     castVarArgs,
     -- * Debugging
-    dumpValue, dumpType, getValueName, annotateValueList
+    dumpValue, dumpType, annotateValueList,
+    getValueName, setValueName, setValueName_
     ) where
+
 import qualified LLVM.FFI.Core as FFI
 import LLVM.Core.Util hiding (Function, BasicBlock, createModule, constString, constStringNul, constVector, constArray, constStruct, getModuleValues, valueHasType)
 import LLVM.Core.CodeGen
@@ -96,6 +99,14 @@ dumpType (Value v) = showTypeOf v >>= putStrLn
 getValueName :: Value a -> IO String
 getValueName (Value a) = getValueNameU a
 
+-- |Set the name of a 'Value'.
+setValueName :: String -> Value a -> IO (Value a)
+setValueName str v@(Value a) = setValueNameU str a >> return v
+
+-- |Set the name of a 'Value'.
+setValueName_ :: String -> Value a -> IO ()
+setValueName_ str (Value a) = setValueNameU str a
+
 -- |Convert a varargs function to a regular function.
 castVarArgs :: (CastVarArgs a b) => Function a -> Function b
 castVarArgs (Value a) = Value a
