@@ -24,7 +24,7 @@ module LLVM.Core.CodeGen(
     withString, withStringNul,
     constVector, constArray, constStruct, constPackedStruct,
     -- * Basic blocks
-    BasicBlock(..), newBasicBlock, newNamedBasicBlock, defineBasicBlock, createBasicBlock, getCurrentBasicBlock,
+    BasicBlock(..), newBasicBlock, newNamedBasicBlock, defineBasicBlock, createBasicBlock, createNamedBasicBlock, getCurrentBasicBlock,
     fromLabel, toLabel,
     -- * Misc
     withCurrentBuilder
@@ -143,10 +143,10 @@ instance IsConstFields () where
     constFieldsOf _ = []
 
 constEnum :: (Enum a) => FFI.TypeRef -> a -> ConstValue a
-constEnum t i = ConstValue $ FFI.constInt t (fromIntegral $ fromEnum i) 0
+constEnum t i = ConstValue $ FFI.constInt t (fromIntegral $ fromEnum i) False
 
 constI :: (IsInteger a, Integral a) => a -> ConstValue a
-constI i = ConstValue $ FFI.constInt (typeRef i) (fromIntegral i) (fromIntegral $ fromEnum $ isSigned i)
+constI i = ConstValue $ FFI.constInt (typeRef i) (fromIntegral i) (isSigned i)
 
 constF :: (IsFloating a, Real a) => a -> ConstValue a
 constF i = ConstValue $ FFI.constReal (typeRef i) (realToFrac i)
@@ -303,6 +303,12 @@ createBasicBlock = do
     defineBasicBlock b
     return b
 
+createNamedBasicBlock :: String -> CodeGenFunction r BasicBlock
+createNamedBasicBlock name = do
+    b <- newNamedBasicBlock name
+    defineBasicBlock b
+    return b
+
 newBasicBlock :: CodeGenFunction r BasicBlock
 newBasicBlock = genFSym >>= newNamedBasicBlock
 
@@ -409,7 +415,7 @@ newNamedGlobal isConst linkage name = do
     modul <- getModule
     let typ = typeRef (undefined :: a)
     liftIO $ liftM Value $ do g <- U.addGlobal modul linkage name typ
-    	     	   	      when isConst $ FFI.setGlobalConstant g 1
+    	     	   	      when isConst $ FFI.setGlobalConstant g True
 			      return g
 
 -- | Create a new global variable.
@@ -490,7 +496,7 @@ string n s = do
     name <- genMSym "str"
     let typ = FFI.arrayType (typeRef (undefined :: Word8)) (fromIntegral n)
     liftIO $ liftM Value $ do g <- U.addGlobal modul InternalLinkage name typ
-    	     	   	      FFI.setGlobalConstant g 1
+    	     	   	      FFI.setGlobalConstant g True
 			      FFI.setInitializer g s
 			      return g
 
