@@ -3,7 +3,6 @@ module LLVM.Core.Data(IntN(..), WordN(..), FP128(..),
        		      Array(..), Vector(..), Ptr, Label, Struct(..), PackedStruct(..)) where
 import Data.Typeable
 import Foreign.Ptr(Ptr)
-import Data.TypeLevel
 
 -- TODO:
 -- Make instances IntN, WordN to actually do the right thing.
@@ -12,12 +11,12 @@ import Data.TypeLevel
 
 -- |Variable sized signed integer.
 -- The /n/ parameter should belong to @PosI@.
-newtype (Pos n) => IntN n = IntN Integer
+newtype IntN n = IntN Integer
     deriving (Show, Typeable)
 
 -- |Variable sized unsigned integer.
 -- The /n/ parameter should belong to @PosI@.
-newtype (Pos n) => WordN n = WordN Integer
+newtype WordN n = WordN Integer
     deriving (Show, Typeable)
 
 -- |128 bit floating point.
@@ -25,7 +24,7 @@ newtype FP128 = FP128 Rational
     deriving (Show, Typeable)
 
 -- |Fixed sized arrays, the array size is encoded in the /n/ parameter.
-newtype (Nat n) => Array n a = Array [a]
+newtype Array n a = Array [a]
     deriving (Show, Typeable)
 
 -- |Fixed sized vector, the array size is encoded in the /n/ parameter.
@@ -41,4 +40,3 @@ newtype Struct a = Struct a
     deriving (Show, Typeable)
 newtype PackedStruct a = PackedStruct a
     deriving (Show, Typeable)
-
