@@ -35,7 +35,7 @@ module LLVM.Core.Instructions(
     free,
     load,
     store,
-    getElementPtr, getElementPtr0,
+    getElementPtr, getElementPtr0, unsafeGetElementPtr,
     -- * Conversions
     trunc, zext, sext,
     fptrunc, fpext,
@@ -45,12 +45,13 @@ module LLVM.Core.Instructions(
     bitcast, bitcastUnify,
     -- * Comparison
     CmpPredicate(..), IntPredicate(..), FPPredicate(..),
-    CmpRet,
+    CmpRet(..),
     cmp, pcmp, icmp, fcmp,
     select,
     -- * Other
     phi, addPhiInputs,
     call, callWithConv,
+    sizeOf, alignOf,
 
     -- * Classes and types
     Terminate,
@@ -65,10 +66,18 @@ import Data.Int
 import Data.Word
 import Data.Map(fromList, (!))
 import Foreign.Ptr (FunPtr, )
-import Foreign.C(CInt, CUInt)
+import Foreign.C(CUInt)
 import Data.TypeLevel((:<:), (:>:), (:==:), (:*),
           D0, D1, D2, D3, D4, D5, D6, D7, D8, D9, d1, toNum, Succ)
 import qualified LLVM.FFI.Core as FFI
+import LLVM.FFI.Core
+    ( IntPredicate(..)
+    , toIntPredicate
+    , fromIntPredicate
+    , FPPredicate(..)
+    , toFPPredicate
+    , fromFPPredicate
+    )
 import LLVM.Core.Data
 import LLVM.Core.Type
 import LLVM.Core.CodeGenMonad
@@ -688,51 +697,6 @@ fpFromCmpPredicate p =
       CmpLT -> FPOLT
       CmpLE -> FPOLE
 
-
-data IntPredicate =
-    IntEQ                       -- ^ equal
-  | IntNE                       -- ^ not equal
-  | IntUGT                      -- ^ unsigned greater than
-  | IntUGE                      -- ^ unsigned greater or equal
-  | IntULT                      -- ^ unsigned less than
-  | IntULE                      -- ^ unsigned less or equal
-  | IntSGT                      -- ^ signed greater than
-  | IntSGE                      -- ^ signed greater or equal
-  | IntSLT                      -- ^ signed less than
-  | IntSLE                      -- ^ signed less or equal
-    deriving (Eq, Ord, Enum, Show, Typeable)
-
-fromIntPredicate :: IntPredicate -> CInt
-fromIntPredicate p = fromIntegral (fromEnum p + 32)
-
-toIntPredicate :: Int -> IntPredicate
-toIntPredicate p = toEnum $ fromIntegral p - 32
-
-data FPPredicate =
-    FPFalse           -- ^ Always false (always folded)
-  | FPOEQ             -- ^ True if ordered and equal
-  | FPOGT             -- ^ True if ordered and greater than
-  | FPOGE             -- ^ True if ordered and greater than or equal
-  | FPOLT             -- ^ True if ordered and less than
-  | FPOLE             -- ^ True if ordered and less than or equal
-  | FPONE             -- ^ True if ordered and operands are unequal
-  | FPORD             -- ^ True if ordered (no nans)
-  | FPUNO             -- ^ True if unordered: isnan(X) | isnan(Y)
-  | FPUEQ             -- ^ True if unordered or equal
-  | FPUGT             -- ^ True if unordered or greater than
-  | FPUGE             -- ^ True if unordered, greater than, or equal
-  | FPULT             -- ^ True if unordered or less than
-  | FPULE             -- ^ True if unordered, less than, or equal
-  | FPUNE             -- ^ True if unordered or not equal
-  | FPT               -- ^ Always true (always folded)
-    deriving (Eq, Ord, Enum, Show, Typeable)
-
-fromFPPredicate :: FPPredicate -> CInt
-fromFPPredicate p = fromIntegral (fromEnum p)
-
-toFPPredicate :: Int -> FPPredicate
-toFPPredicate p = toEnum $ fromIntegral p
-
 -- |Acceptable operands to comparison instructions.
 class CmpOp a b c d | a b -> c where
     cmpop :: FFIBinOp -> a -> b -> CodeGenFunction r (Value d)
@@ -1014,8 +978,8 @@ free ptr = do
 -- | If we want to export that, then we should have a Size type
 -- This is the official implementation,
 -- but it suffers from the ptrtoint(gep) bug.
-_sizeOf :: forall a r s . (IsSized a s) => a -> CodeGenFunction r (Value Word64)
-_sizeOf a =
+sizeOf :: forall a r s . (IsSized a s) => a -> CodeGenFunction r (Value Word64)
+sizeOf a =
     liftIO $ liftM Value $
     FFI.sizeOf (typeRef a)
 
@@ -1145,13 +1109,9 @@ instance (GetField as i b, Succ i i') => GetField (a, as) i' b
 -- (This is without a doubt the most confusing LLVM instruction, but the types help.)
 getElementPtr :: forall a o i n r . (GetElementPtr o i n, IsIndexArg a) =>
                  Value (Ptr o) -> (a, i) -> CodeGenFunction r (Value (Ptr n))
-getElementPtr (Value ptr) (a, ixs) =
+getElementPtr val (a, ixs) =
     let ixl = getArg a : getIxList (undefined :: o) ixs in
-    liftM Value $
-    withCurrentBuilder $ \ bldPtr ->
-      U.withArrayLen ixl $ \ idxLen idxPtr ->
-        U.withEmptyCString $
-          FFI.buildGEP bldPtr ptr idxPtr (fromIntegral idxLen)
+    getElementPtrFromValues val ixl
 
 -- | Like getElementPtr, but with an initial index that is 0.
 -- This is useful since any pointer first need to be indexed off the pointer, and then into
@@ -1160,6 +1120,26 @@ getElementPtr0 :: (GetElementPtr o i n) =>
                   Value (Ptr o) -> i -> CodeGenFunction r (Value (Ptr n))
 getElementPtr0 p i = getElementPtr p (0::Word32, i)
 
+-- | Call getelementptr directly with a list of indexes.
+-- This is should be used only if the runtime type is not yet known.
+-- If the indexes and return type are incorrect this function may segfault
+-- in LLVMBuildGEP, or assert if debug assertions are enabled.
+unsafeGetElementPtr :: forall o i n r . IsIndexArg i =>
+                       Value (Ptr o) -> [i] -> CodeGenFunction r (Value (Ptr n))
+unsafeGetElementPtr val i =
+    let ixl = map getArg i in
+    getElementPtrFromValues val ixl
+
+-- | Internal function for emitting a GEP
+getElementPtrFromValues :: forall o n r .
+                           Value (Ptr o) -> [FFI.ValueRef] -> CodeGenFunction r (Value n)
+getElementPtrFromValues (Value ptr) ixl =
+    liftM Value $
+    withCurrentBuilder $ \ bldPtr ->
+      U.withArrayLen ixl $ \ idxLen idxPtr ->
+        U.withEmptyCString $
+          FFI.buildGEP bldPtr ptr idxPtr (fromIntegral idxLen)
+
 --------------------------------------
 {-
 instance (IsConst a) => Show (ConstValue a) -- XXX
