@@ -31,7 +31,7 @@ module LLVM.Core.Util(
     CString, withArrayLen,
     withEmptyCString,
     functionType, buildEmptyPhi, addPhiIns,
-    showTypeOf, getValueNameU, getObjList, annotateValueList, isConstant,
+    showTypeOf, getValueNameU, setValueNameU, getObjList, annotateValueList, isConstant,
     -- * Transformation passes
     addCFGSimplificationPass, addConstantPropagationPass, addDemoteMemoryToRegisterPass,
     addGVNPass, addInstructionCombiningPass, addPromoteMemoryToRegisterPass, addReassociatePass,
@@ -46,7 +46,6 @@ import Foreign.Ptr (Ptr, nullPtr)
 import Foreign.Marshal.Array (withArrayLen, withArray, allocaArray, peekArray)
 import Foreign.Marshal.Alloc (alloca)
 import Foreign.Storable (Storable(..))
-import Foreign.Marshal.Utils (fromBool)
 import System.IO.Unsafe (unsafePerformIO)
 
 import qualified LLVM.FFI.Core as FFI
@@ -61,14 +60,13 @@ type Type = FFI.TypeRef
 functionType :: Bool -> Type -> [Type] -> Type
 functionType varargs retType paramTypes = unsafePerformIO $
     withArrayLen paramTypes $ \ len ptr ->
-        return $ FFI.functionType retType ptr (fromIntegral len)
-	       	 		  (fromBool varargs)
+        return $ FFI.functionType retType ptr (fromIntegral len) varargs
 
 -- unsafePerformIO just to wrap the non-effecting withArrayLen call
 structType :: [Type] -> Bool -> Type
 structType types packed = unsafePerformIO $
     withArrayLen types $ \ len ptr ->
-        return $ FFI.structType ptr (fromIntegral len) (if packed then 1 else 0)
+        return $ FFI.structType ptr (fromIntegral len) packed
 
 --------------------------------------
 -- Handle modules
@@ -106,7 +104,7 @@ writeBitcodeToFile name mdl =
     withCString name $ \ namePtr ->
       withModule mdl $ \ mdlPtr -> do
         rc <- FFI.writeBitcodeToFile mdlPtr namePtr
-        when (rc /= 0) $
+        when (rc /= False) $
           ioError $ userError $ "writeBitcodeToFile: return code " ++ show rc
         return ()
 
@@ -118,13 +116,13 @@ readBitcodeFromFile name =
       alloca $ \ modPtr ->
       alloca $ \ errStr -> do
         rrc <- FFI.createMemoryBufferWithContentsOfFile namePtr bufPtr errStr
-        if rrc /= 0 then do
+        if rrc /= False then do
             msg <- peek errStr >>= peekCString
             ioError $ userError $ "readBitcodeFromFile: read return code " ++ show rrc ++ ", " ++ msg
          else do
             buf <- peek bufPtr
             prc <- FFI.parseBitcode buf modPtr errStr
-	    if prc /= 0 then do
+	    if prc /= False then do
                 msg <- peek errStr >>= peekCString
                 ioError $ userError $ "readBitcodeFromFile: parse return code " ++ show prc ++ ", " ++ msg
              else do
@@ -160,11 +158,14 @@ showType' p = do
     pk <- FFI.getTypeKind p
     case pk of
         FFI.VoidTypeKind -> return "()"
+        FFI.HalfTypeKind -> return "Half"
 	FFI.FloatTypeKind -> return "Float"
 	FFI.DoubleTypeKind -> return "Double"
 	FFI.X86_FP80TypeKind -> return "X86_FP80"
 	FFI.FP128TypeKind -> return "FP128"
 	FFI.PPC_FP128TypeKind -> return "PPC_FP128"
+        FFI.X86_MMXTypeKind -> return "X86_MMX"
+        FFI.MetadataTypeKind -> return "Metadata"
 	FFI.LabelTypeKind -> return "Label"
 	FFI.IntegerTypeKind -> do w <- FFI.getIntTypeWidth p; return $ "(IntN " ++ show w ++ ")"
 	FFI.FunctionTypeKind -> do
@@ -179,7 +180,6 @@ showType' p = do
 	FFI.StructTypeKind -> return "(Struct ...)"
 	FFI.ArrayTypeKind -> do n <- FFI.getArrayLength p; t <- FFI.getElementType p >>= showType'; return $ "(Array " ++ show n ++ " " ++ t ++ ")"
 	FFI.PointerTypeKind -> do t <- FFI.getElementType p >>= showType'; return $ "(Ptr " ++ t ++ ")"
-	FFI.OpaqueTypeKind -> return "Opaque"
 	FFI.VectorTypeKind -> do n <- FFI.getVectorSize p; t <- FFI.getElementType p >>= showType'; return $ "(Vector " ++ show n ++ " " ++ t ++ ")"
 
 --------------------------------------
@@ -274,7 +274,7 @@ addGlobal modul linkage name typ =
 constStringInternal :: Bool -> String -> (Value, Int)
 constStringInternal nulTerm s = unsafePerformIO $
     withCStringLen s $ \(sPtr, sLen) ->
-      return (FFI.constString sPtr (fromIntegral sLen) (fromBool (not nulTerm)), sLen)
+      return (FFI.constString sPtr (fromIntegral sLen) (not nulTerm), sLen)
 
 constString :: String -> (Value, Int)
 constString = constStringInternal False
@@ -403,14 +403,14 @@ addReassociatePass pm = withPassManager pm FFI.addReassociatePass
 addTargetData :: FFI.TargetDataRef -> PassManager -> IO ()
 addTargetData td pm = withPassManager pm $ FFI.addTargetData td
 
-runFunctionPassManager :: PassManager -> Function -> IO Int
-runFunctionPassManager pm fcn = liftM fromIntegral $ withPassManager pm $ \ pmref -> FFI.runFunctionPassManager pmref fcn
+runFunctionPassManager :: PassManager -> Function -> IO Bool
+runFunctionPassManager pm fcn = withPassManager pm $ \ pmref -> FFI.runFunctionPassManager pmref fcn
 
-initializeFunctionPassManager :: PassManager -> IO Int
-initializeFunctionPassManager pm = liftM fromIntegral $ withPassManager pm FFI.initializeFunctionPassManager
+initializeFunctionPassManager :: PassManager -> IO Bool
+initializeFunctionPassManager pm = withPassManager pm FFI.initializeFunctionPassManager
 
-finalizeFunctionPassManager :: PassManager -> IO Int
-finalizeFunctionPassManager pm = liftM fromIntegral $ withPassManager pm FFI.finalizeFunctionPassManager
+finalizeFunctionPassManager :: PassManager -> IO Bool
+finalizeFunctionPassManager pm = withPassManager pm FFI.finalizeFunctionPassManager
 
 --------------------------------------
 
@@ -432,7 +432,7 @@ constArray t n xs = unsafePerformIO $ do
 constStruct :: [Value] -> Bool -> Value
 constStruct xs packed = unsafePerformIO $ do
     withArrayLen xs $ \ len ptr ->
-        return $ FFI.constStruct ptr (fromIntegral len) (if packed then 1 else 0)
+        return $ FFI.constStruct ptr (fromIntegral len) packed
 
 --------------------------------------
 
@@ -443,6 +443,11 @@ getValueNameU a = do
     str <- peekCString cs
     if str == "" then return (show a) else return str
 
+setValueNameU :: String -> Value -> IO ()
+setValueNameU str a = do
+    withCString str $ \ strPtr ->
+        FFI.setValueName a strPtr
+
 getObjList :: (t1 -> (t2 -> IO [Ptr a]) -> t) -> (t2 -> IO (Ptr a))
            -> (Ptr a -> IO (Ptr a)) -> t1 -> t
 getObjList withF firstF nextF obj = do
@@ -460,9 +465,7 @@ annotateValueList vs = do
   return $ zip names vs
 
 isConstant :: Value -> IO Bool
-isConstant v = do
-  isC <- FFI.isConstant v
-  if isC == 0 then return False else return True
+isConstant = FFI.isConstant
 
 isIntrinsic :: Value -> IO Bool
 isIntrinsic v = do
