@@ -75,9 +75,7 @@ http://llvm.org/bugs/show_bug.cgi?id=6394
 
     moduleModified <- FFI.runPassManager passes m
 
-    return $
-       toEnum (fromIntegral moduleModified) ||
-       toEnum (fromIntegral functionsModified)
+    return $ moduleModified || functionsModified
 
 -- tools/opt/opt.cpp: AddOptimizationPasses
 addOptimizationPasses :: FFI.PassManagerRef -> FFI.PassManagerRef -> Int -> IO ()
